unit FTPGeneralFrameUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, Buttons, Leds, Registry, FTPUtils,
  ImgList, StrUtils;

type
  TFTPGeneralFrame = class(TFrame)
    GroupBox1: TGroupBox;
    Panel1: TPanel;
    Panel2: TPanel;
    Edit1: TEdit;
    Edit2: TEdit;
    UpDown1: TUpDown;
    UpDown2: TUpDown;
    Panel3: TPanel;
    OnOffLed1: TOnOffLed;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    GroupBox2: TGroupBox;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    GroupBox3: TGroupBox;
    RichEdit1: TRichEdit;
    Panel4: TPanel;
    ComboBox1: TComboBox;
    ImageList1: TImageList;
    procedure CheckBox3Click(Sender: TObject);
    procedure ComboBox1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure ComboBox1Change(Sender: TObject);
  private
  public
    constructor Create(AOwner:TComponent);override;

    procedure ConnectServer;
    procedure DisconnectServer;
  end;

implementation

uses
  MainFormUnit, LogFormUnit;

{$R *.dfm}

{ TFTPGeneralFrame }

procedure TFTPGeneralFrame.ConnectServer;
var
  a:Integer;
  s:string;
begin
  if not MainForm.IdFTPServer1.Active then begin
    with MainForm.FTPAccountsFrame1.UserManager do begin
      s:=AnsiReplaceStr(ExtractFilePath(Application.ExeName),'\','/');
      for a:=1 to GroupCount-1 do
        if Group[a].FolderPermission[s]>=ftpRead then
          case MessageDlg('The group "'+Group[a].GroupName+'" has read permissions to the folder"'+s+
                          '", which is the Little FTP Server application directory.'#13#13+
                          'This means that the users in this group can read the content of this folder, '+
                          'which contains the server log and the passwords of all users, and is a high critical security risk.'#13#13+
                          'Do you want to reset these permissions to None (HIGHLY RECOMMENDED)?',mtWarning,mbYesNoCancel,0) of
            mrYes:begin
              Group[a].FolderPermission[s]:=ftpNone;
              //MainForm.FTPAccountsFrame1.UpdateGUI;
            end;
            mrCancel:Exit;
        end;
    end;
    MainForm.IdFTPServer1.DefaultPort:=UpDown1.Position;
    MainForm.IdFTPServer1.DefaultDataPort:=UpDown2.Position;
    MainForm.IdFTPServer1.Greeting.Text.Assign(RichEdit1.Lines);
    if MainForm.IdFTPServer1.Greeting.Text.Text='' then begin
      MainForm.IdFTPServer1.Greeting.Text.Text:='FTP server ready, proceed';
      RichEdit1.Text:='FTP server ready, proceed';
    end;
    MainForm.IdFTPServer1.Active:=True;
    MainForm.FTPAccountsFrame1.UserManager.AddLogEntry('Local',lConnect,'Admin','Server online');
    OnOffLed1.Active:=True;
    MainForm.Action4.Visible:=True;
    MainForm.Action3.Visible:=False;
    Panel3.Caption:='Server state: ON';
    MainForm.Timer1.Enabled:=True;
    UpDown1.Enabled:=False;
    UpDown2.Enabled:=False;
    Edit1.Enabled:=False;
    Edit2.Enabled:=False;
    RichEdit1.Enabled:=False;
    MainForm.TabSheet2.TabVisible:=False;
    MainForm.TabSheet3.TabVisible:=False;
    MainForm.Action6.Enabled:=False;
    MainForm.Action1.Enabled:=False;
    MainForm.StatusBar1.Panels[0].Text:='Server status: ON';
    MainForm.OnOffLed1.Active:=True;
  end;
end;

constructor TFTPGeneralFrame.Create(AOwner: TComponent);
begin
  inherited;
end;

procedure TFTPGeneralFrame.DisconnectServer;
var
  l:TList;
  t:Boolean;
begin
  if MainForm.IdFTPServer1.Active then begin
    l:=LogForm.UserList.LockList;
    try
      t:=(l.Count>0) and
         (MessageDlg('There are actually '+IntToStr(l.Count)+' user(s) connected, are you sure you want to shut down the server?',
                     mtConfirmation,[mbYes,mbNo],0)=mrNo);
    finally
      LogForm.UserList.UnlockList;
    end;
    if t then
      Exit;
    MainForm.IdFTPServer1.Active:=False;
    MainForm.FTPAccountsFrame1.UserManager.AddLogEntry('Local',lDisconnect,'Admin','Server offline');
    OnOffLed1.Active:=False;
    MainForm.Action4.Visible:=False;
    MainForm.Action3.Visible:=True;
    Panel3.Caption:='Server state: OFF';
    if MainForm.NID.cbSize<>0 then
      MainForm.Perform(WM_SYSTRAYICON,0,WM_LBUTTONDBLCLK);
    MainForm.Timer1.Enabled:=False;
    UpDown1.Enabled:=True;
    UpDown2.Enabled:=True;
    Edit1.Enabled:=True;
    Edit2.Enabled:=True;
    RichEdit1.Enabled:=True;
    MainForm.TabSheet2.TabVisible:=True;
    MainForm.TabSheet3.TabVisible:=True;
    MainForm.Action6.Enabled:=True;
    MainForm.Action1.Enabled:=True;
    MainForm.StatusBar1.Panels[0].Text:='Server status: OFF';
    MainForm.OnOffLed1.Active:=False;
  end;
end;

procedure TFTPGeneralFrame.CheckBox3Click(Sender: TObject);
var
  Reg:TRegistry;
begin
  Reg:=TRegistry.Create;
  Reg.RootKey:=HKEY_LOCAL_MACHINE;
  Reg.OpenKey('\Software\Microsoft\Windows\CurrentVersion\Run\',True);
  if CheckBox3.Checked then begin
    Reg.WriteString('Little FTP server',Application.ExeName+' -AutoLaunch');
  end else
    Reg.DeleteValue('Little FTP server');
  Reg.CloseKey;
  Reg.Destroy;
end;

procedure TFTPGeneralFrame.ComboBox1DrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  with ComboBox1,ComboBox1.Canvas do begin
    if odSelected in State then
      Brush.Color:=clHighlight;
    FillRect(Rect);
    ImageList1.Draw(Canvas,Rect.Left+1,Rect.Top,Index);
    TextOut(Rect.Left+21,Rect.Top+2,Items[Index]);
  end;
end;

procedure TFTPGeneralFrame.ComboBox1Change(Sender: TObject);
begin
  MainForm.FTPAccountsFrame1.UserManager.PermanentDelete:=ComboBox1.ItemIndex=1;
end;

end.
