unit FTPTrafficFrameUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ShellAPI, ComCtrls;

const
  GTrafficIntervals=400;
  GTrafficLengths:array[0..6] of Double=(10,60,600,3600,3600*24,3600*24*7,3600*24*30);

type
  TFTPTrafficFrame = class(TFrame)
    Timer1: TTimer;
    Panel1: TPanel;
    ComboBox1: TComboBox;
    Panel2: TPanel;
    PaintBox1: TPaintBox;
    Panel3: TPanel;
    Panel4: TPanel;
    Edit1: TEdit;
    UpDown1: TUpDown;
    Edit2: TEdit;
    UpDown2: TUpDown;
    procedure Timer1Timer(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure UpDown2Click(Sender: TObject; Button: TUDBtnType);
  private
    FFirstTick:Double;
    FTrafficData:array[0..1,0..6,0..GTrafficIntervals] of Double;
    FTrafficIDs:array[0..6] of Int64;
    FLastSent,FLastReceived:Int64;
    FSendRate,FReceiveRate:Double;
  public
    constructor Create(AOwner:TComponent);override;

    property SendRate:Double read FSendRate;
    property ReceiveRate:Double read FReceiveRate;
  end;

function ShortTrafficToStr(Value:Double):string;
function LongTrafficToStr(Value:Double):string;

implementation

uses MainFormUnit, FTPUtils;

function ShortTrafficToStr(Value:Double):string;
begin
  if Value<1024 then
    Result:=FormatFloat('0.00',Value)+' o'
  else begin
    if Value<1024*1024 then
      Result:=FormatFloat('0.00',Value/1024)+' Ko'
    else begin
      if Value<1024*1024*1024 then
        Result:=FormatFloat('0.00',Value/(1024*1024))+' Mo'
      else
        Result:=FormatFloat('0.00',Value/(1024*1024*1024))+' Go';
    end;
  end;
end;

function LongTrafficToStr(Value:Double):string;
begin
  if Value<1024 then
    Result:=FormatFloat('0.00',Value)+' Octets'
  else begin
    if Value<1024*1024 then
      Result:=FormatFloat('0.00',Value/1024)+' Kilo-Octets'
    else begin
      if Value<1024*1024*1024 then
        Result:=FormatFloat('0.00',Value/(1024*1024))+' Mega-Octets'
      else
        Result:=FormatFloat('0.00',Value/(1024*1024*1024))+' Giga-Octets';
    end;
  end;
end;

function ShortTrafficSuffix(Value:Double):string;
begin
  if Value<1024 then
    Result:='o'
  else begin
    if Value<1024*1024 then
      Result:='Ko'
    else begin
      if Value<1024*1024*1024 then
        Result:='Mo'
      else
        Result:='Go';
    end;
  end;
end;

{$R *.dfm}

procedure TFTPTrafficFrame.Timer1Timer(Sender: TObject);
var
  a,c:Integer;
  b:Double;
  s:string;

  procedure MakeOffset(ID:Integer;Offset:Integer);
  var
    a:Integer;
  begin
    if Offset=0 then 
      Exit;
    if Offset>=GTrafficIntervals then
      Offset:=GTrafficIntervals-1;
    for a:=0 to GTrafficIntervals-Offset do begin
      FTrafficData[0,ID,a]:=FTrafficData[0,ID,a+Offset];
      FTrafficData[1,ID,a]:=FTrafficData[1,ID,a+Offset];
    end;
  end;

  procedure MakeNewData(ID:Integer;Offset:Integer);
  var
    a,b:Integer;
    s1,s2,u,v1,v2:Double;
  begin
    if Offset=0 then
      Exit;
    if Offset>=GTrafficIntervals then
      Offset:=GTrafficIntervals-1;
    if ID=0 then begin
      MainForm.FTPAccountsFrame1.UserManager.Lock;
      try
        s1:=(MainForm.FTPAccountsFrame1.UserManager.SentData-FLastSent)/(GTrafficLengths[0]*Offset/GTrafficIntervals);
        s2:=(MainForm.FTPAccountsFrame1.UserManager.ReceivedData-FLastReceived)/(GTrafficLengths[0]*Offset/GTrafficIntervals);
        FLastSent:=MainForm.FTPAccountsFrame1.UserManager.SentData;
        FLastReceived:=MainForm.FTPAccountsFrame1.UserManager.ReceivedData;
      finally
        MainForm.FTPAccountsFrame1.UserManager.UnLock;
      end;
      v1:=0.7;
      if s1<1000 then
        v1:=0;
      s1:=(1-v1)*s1+v1*FSendRate;
      v2:=0.7;
      if s2<1000 then
        v2:=0;
      s2:=(1-v2)*s2+v2*FReceiveRate;
      FSendRate:=s1;
      FReceiveRate:=s2;
    end else begin
      s1:=0;
      s2:=0;
      b:=Round(GTrafficLengths[ID]/GTrafficLengths[ID-1]);
      for a:=GTrafficIntervals-b+1 to GTrafficIntervals do begin
        s1:=s1+FTrafficData[0,ID-1,a];
        s2:=s2+FTrafficData[1,ID-1,a];
      end;
      s1:=s1/b;
      s2:=s2/b;
    end;
    for a:=GTrafficIntervals-Offset+1 to GTrafficIntervals do begin
      u:=(a-GTrafficIntervals+Offset)/Offset;
      FTrafficData[0,ID,a]:=u*s1+(1-u)*FTrafficData[0,ID,GTrafficIntervals-Offset];
      FTrafficData[1,ID,a]:=u*s2+(1-u)*FTrafficData[1,ID,GTrafficIntervals-Offset];
    end;
  end;

begin
  b:=Now*3600*24;
  for a:=0 to 6 do begin
    c:=Round((b-FFirstTick)*(GTrafficIntervals+1)/GTrafficLengths[a]);
    MakeOffset(a,c-FTrafficIDs[a]);
    MakeNewData(a,c-FTrafficIDs[a]);
//    if (ComboBox1.ItemIndex=a) and (c-FTrafficIDs[a]>0) then
//      PaintBox1Paint(nil);
    FTrafficIDs[a]:=c;
  end;
  PaintBox1Paint(nil);
  if MainForm.NID.cbSize<>0 then begin
    s:=MainForm.Caption+' (connected)'#13+'U: '+ShortTrafficToStr(FSendRate)+'/s'#13'D: '+ShortTrafficToStr(FReceiveRate)+'/s';
    s:=Copy(s,1,63);
    ZeroMemory(@MainForm.NID.szTip[0],64);
    CopyMemory(@MainForm.NID.szTip[0],@s[1],Length(s));
    Assert(Shell_NotifyIcon(NIM_MODIFY,@MainForm.NID),SysErrorMessage(GetLastError));
  end;
  MainForm.StatusBar1.Panels[1].Text:='Send speed : '+ShortTrafficToStr(FSendRate)+'/s';
  MainForm.StatusBar1.Panels[2].Text:='Receive speed : '+ShortTrafficToStr(FReceiveRate)+'/s';
end;

function GetTransfertUnit(x:Double):Double;
begin
  if x<1024 then
    Result:=1
  else begin
    if x<1024*1024 then
      Result:=1024
    else begin
      if x<1024*1024*1024 then
        Result:=1024*1024
      else
        Result:=1024*1024*1024;
    end;
  end;
end;

function GetUnit(x:Double):Double;
begin
  Result:=Exp(Ln(10)*Trunc(Ln(x)/Ln(10)));
end;

procedure TFTPTrafficFrame.PaintBox1Paint(Sender: TObject);
var
  a,b:Integer;
  x,y,w,h:Integer;
  m,u,v:Double;
  s:string;
begin
  with PaintBox1,PaintBox1.Canvas do begin
    Lock;
    x:=50;
    y:=ClientHeight-10;
    w:=ClientWidth-60;
    h:=-(ClientHeight-40);
    Brush.Color:=0;
    FillRect(ClientRect);
    m:=0;
    for a:=0 to GTrafficIntervals do begin
      if FTrafficData[0,ComboBox1.ItemIndex,a]>m then
        m:=FTrafficData[0,ComboBox1.ItemIndex,a];
      if FTrafficData[1,ComboBox1.ItemIndex,a]>m then
        m:=FTrafficData[1,ComboBox1.ItemIndex,a];
    end;
    if m=0 then
      m:=1;

    s:=LongTrafficToStr(FLastSent)+' sent';
    Font.Color:=clRed;
    TextOut(x+w div 4-TextWidth(s) div 2,5,s);
    s:=LongTrafficToStr(FLastReceived)+' received';
    Font.Color:=clLime;
    TextOut(x+(3*w) div 4-TextWidth(s) div 2,5,s);


    u:=GetTransfertUnit(m/3);
    v:=GetUnit(m/u);
    if m/v<3 then
      v:=v/5;
//    if m/v>15 then
//      v:=v*2;


    Pen.Color:=clWhite;
    Pen.Mode:=pmCopy;
    MoveTo(x-40,y);
    LineTo(x+w,y);
    MoveTo(x,y);
    LineTo(x,y+h);
    MoveTo(x-4,y+h);
    LineTo(x+5,y+h);

    Font.Color:=clWhite;
    if m/u<10 then
      s:=FormatFloat('0.0#',m/u)+' '+ShortTrafficSuffix(u)+'/s'
    else begin
      if m/u<100 then
        s:=FormatFloat('00.#',m/u)+' '+ShortTrafficSuffix(u)+'/s'
      else
        s:=FormatFloat('00#',m/u)+' '+ShortTrafficSuffix(u)+'/s'
    end;
    TextOut(x-TextWidth(s) div 2,10,s);

    b:=Round(0.5+m/(v*u));
    for a:=1 to b-1 do begin
      s:=FormatFloat('0.##',a*v);
      TextOut(40-TextWidth(s),Round(y+h*a*u*v/m)-5,s);
      Pen.Color:=clWhite;
      MoveTo(x-5,Round(y+h*a*u*v/m));
      LineTo(x,Round(y+h*a*u*v/m));
      Pen.Color:=$664444;
      LineTo(x+w,Round(y+h*a*u*v/m));
    end;
    Pen.Color:=$666666;
    MoveTo(x+w,y);
    LineTo(x+w,y+h);

    Pen.Color:=clRed;
    MoveTo(x+1,y-1+Round(FTrafficData[0,ComboBox1.ItemIndex,0]*(h+1)/m));
    for a:=1 to GTrafficIntervals do
      LineTo(x+1+Round((w-1)*a/GTrafficIntervals),y-1+Round(FTrafficData[0,ComboBox1.ItemIndex,a]*(h+1)/m));

    Pen.Color:=clLime;
    Pen.Mode:=pmMerge;
    MoveTo(x+1,y-1+Round(FTrafficData[1,ComboBox1.ItemIndex,0]*(h+1)/m));
    for a:=1 to GTrafficIntervals do
      LineTo(x+1+Round((w-1)*a/GTrafficIntervals),y-1+Round(FTrafficData[1,ComboBox1.ItemIndex,a]*(h+1)/m));
    Unlock;
  end;
end;

procedure TFTPTrafficFrame.ComboBox1Change(Sender: TObject);
begin
  PaintBox1Paint(nil);
end;

constructor TFTPTrafficFrame.Create(AOwner: TComponent);
begin
  inherited;
  FFirstTick:=Now*3600*24;
  Panel1.DoubleBuffered:=True;
end;

procedure TFTPTrafficFrame.UpDown1Click(Sender: TObject;
  Button: TUDBtnType);
begin
  MainForm.FTPAccountsFrame1.UserManager.MaxUploadSpeed:=UpDown1.Position*1024;
end;

procedure TFTPTrafficFrame.UpDown2Click(Sender: TObject;
  Button: TUDBtnType);
begin
  MainForm.FTPAccountsFrame1.UserManager.MaxDownloadSpeed:=UpDown2.Position*1024;
end;

end.
